@@ -20,6 +20,7 @@
 #include "token.h"
 #include "ql.h"
 #include "pat.h"
+#include "module.h"
 #include "snip.h"
 #include <stdio.h>
 #include <stdarg.h>
@@ -93,14 +94,14 @@ grn_rc
 grn_timeval2str(grn_ctx *ctx, grn_timeval *tv, char *buf)
 {
   struct tm *ltm;
-#ifdef WIN32
-  time_t tvsec = (time_t) tv->tv_sec;
-  ltm = localtime(&tvsec);
-#else /* WIN32 */
+#ifdef HAVE_LOCALTIME_R
   struct tm tm;
   time_t t = tv->tv_sec;
   ltm = localtime_r(&t, &tm);
-#endif /* WIN32 */
+#else /* HAVE_LOCALTIME_R */
+  time_t tvsec = (time_t) tv->tv_sec;
+  ltm = localtime(&tvsec);
+#endif /* HAVE_LOCALTIME_R */
   if (!ltm) { SERR("localtime"); }
   snprintf(buf, GRN_TIMEVAL_STR_SIZE - 1, GRN_TIMEVAL_STR_FORMAT,
            ltm->tm_year + 1900, ltm->tm_mon + 1, ltm->tm_mday,
@@ -167,14 +168,13 @@ char *grn_fmalloc_file = NULL;
 int grn_fmalloc_line = 0;
 #endif /* USE_FAIL_MALLOC */
 
-#define N_SEGMENTS 512
+#define GRN_CTX_SEGMENT_SIZE    (1<<22)
+#define GRN_CTX_SEGMENT_MASK    (GRN_CTX_SEGMENT_SIZE - 1)
 
-#define SEGMENT_SIZE    (1<<22)
-#define SEGMENT_MASK    (SEGMENT_SIZE - 1)
-
-#define SEGMENT_WORD    (1<<31)
-#define SEGMENT_VLEN    (1<<30)
-#define SEGMENT_LIFO    (1<<29)
+#define GRN_CTX_SEGMENT_WORD    (1<<31)
+#define GRN_CTX_SEGMENT_VLEN    (1<<30)
+#define GRN_CTX_SEGMENT_LIFO    (1<<29)
+#define GRN_CTX_SEGMENT_DIRTY   (1<<28)
 
 #ifdef USE_DYNAMIC_MALLOC_CHANGE
 static void
@@ -226,29 +226,30 @@ grn_ctx_loader_clear(grn_ctx *ctx)
   grn_loader_init(loader);
 }
 
+#define IMPL_SIZE ((sizeof(struct _grn_ctx_impl) + (grn_pagesize - 1)) & ~(grn_pagesize - 1))
+
 static void
 grn_ctx_impl_init(grn_ctx *ctx)
 {
-  if (!(ctx->impl = GRN_MALLOC(sizeof(struct _grn_ctx_impl)))) { return; }
-  if (!(ctx->impl->segs = grn_io_anon_map(ctx, &ctx->impl->mi,
-                                          sizeof(grn_io_mapinfo) * N_SEGMENTS))) {
-    GRN_FREE(ctx->impl);
+
+  grn_io_mapinfo mi;
+  if (!(ctx->impl = grn_io_anon_map(ctx, &mi, IMPL_SIZE))) {
     ctx->impl = NULL;
     return;
   }
 #ifdef USE_DYNAMIC_MALLOC_CHANGE
   grn_ctx_impl_init_malloc(ctx);
 #endif
-  if (!(ctx->impl->values = grn_array_create(ctx, NULL, sizeof(grn_tmp_db_obj),
-                                             GRN_ARRAY_TINY))) {
-    grn_io_anon_unmap(ctx, &ctx->impl->mi, sizeof(grn_io_mapinfo) * N_SEGMENTS);
-    GRN_FREE(ctx->impl);
-    ctx->impl = NULL;
-    return;
-  }
   ctx->impl->encoding = ctx->encoding;
   ctx->impl->lifoseg = -1;
   ctx->impl->currseg = -1;
+  if (!(ctx->impl->values = grn_array_create(ctx, NULL, sizeof(grn_tmp_db_obj),
+                                             GRN_ARRAY_TINY))) {
+    grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
+    ctx->impl = NULL;
+    return;
+  }
+  CRITICAL_SECTION_INIT(ctx->impl->lock);
   ctx->impl->db = NULL;
 
   ctx->impl->expr_vars = grn_hash_create(ctx, NULL, sizeof(grn_id), sizeof(grn_expr_vars), 0);
@@ -426,20 +427,6 @@ grn_ctx_fin(grn_ctx *ctx)
     }
     rc = grn_obj_close(ctx, ctx->impl->outbuf);
     rc = grn_bulk_fin(ctx, &ctx->impl->subbuf);
-    {
-      int i;
-      grn_io_mapinfo *mi;
-      for (i = 0, mi = ctx->impl->segs; i < N_SEGMENTS; i++, mi++) {
-        if (mi->map) {
-          if (mi->count & SEGMENT_VLEN) {
-            grn_io_anon_unmap(ctx, mi, mi->nref * grn_pagesize);
-          } else {
-            grn_io_anon_unmap(ctx, mi, SEGMENT_SIZE);
-          }
-        }
-      }
-    }
-    grn_io_anon_unmap(ctx, &ctx->impl->mi, sizeof(grn_io_mapinfo) * N_SEGMENTS);
     {
       uint32_t i;
       grn_expr_var *v;
@@ -452,7 +439,26 @@ grn_ctx_fin(grn_ctx *ctx)
       });
     }
     grn_hash_close(ctx, ctx->impl->expr_vars);
-    GRN_FREE(ctx->impl);
+    {
+      int i;
+      grn_io_mapinfo *mi;
+      for (i = 0, mi = ctx->impl->segs; i < GRN_CTX_N_SEGMENTS; i++, mi++) {
+        if (mi->map) {
+          //GRN_LOG(ctx, GRN_LOG_NOTICE, "unmap in ctx_fin(%d,%d,%d)", i, (mi->count & GRN_CTX_SEGMENT_MASK), mi->nref);
+          if (mi->count & GRN_CTX_SEGMENT_VLEN) {
+            grn_io_anon_unmap(ctx, mi, mi->nref * grn_pagesize);
+          } else {
+            grn_io_anon_unmap(ctx, mi, GRN_CTX_SEGMENT_SIZE);
+          }
+        }
+      }
+    }
+    {
+      grn_io_mapinfo mi;
+      mi.map = (void *)ctx->impl;
+      grn_io_anon_unmap(ctx, &mi, IMPL_SIZE);
+    }
+    CRITICAL_SECTION_FIN(ctx->impl->lock);
   }
   ctx->stat = GRN_CTX_FIN;
   return rc;
@@ -618,10 +624,6 @@ grn_init(void)
     grn_fmalloc_line = atoi(getenv("GRN_FMALLOC_LINE"));
   }
 #endif /* USE_FAIL_MALLOC */
-  if ((rc = grn_token_init())) {
-    GRN_LOG(ctx, GRN_LOG_ALERT, "grn_token_init failed (%d)", rc);
-    return rc;
-  }
   if ((rc = grn_com_init())) {
     GRN_LOG(ctx, GRN_LOG_ALERT, "grn_com_init failed (%d)", rc);
     return rc;
@@ -635,6 +637,14 @@ grn_init(void)
     GRN_LOG(ctx, GRN_LOG_ALERT, "io initialize failed (%d)", rc);
     return rc;
   }
+  if ((rc = grn_modules_init())) {
+    GRN_LOG(ctx, GRN_LOG_ALERT, "modules initialize failed (%d)", rc);
+    return rc;
+  }
+  if ((rc = grn_token_init())) {
+    GRN_LOG(ctx, GRN_LOG_ALERT, "grn_token_init failed (%d)", rc);
+    return rc;
+  }
   /*
   if ((rc = grn_index_init())) {
     GRN_LOG(ctx, GRN_LOG_ALERT, "index initialize failed (%d)", rc);
@@ -689,9 +699,10 @@ grn_fin(void)
     }
   }
   grn_cache_fin();
+  grn_token_fin();
+  grn_modules_fin();
   grn_io_fin();
   grn_ctx_fin(ctx);
-  grn_token_fin();
   grn_com_fin();
   GRN_LOG(ctx, GRN_LOG_NOTICE, "grn_fin (%d)", alloc_count);
   grn_logger_fin();
@@ -808,7 +819,7 @@ grn_ctx_qe_exec_uri(grn_ctx *ctx, const char *path, uint32_t path_len)
   } else {
     grn_obj key;
     const char *g, *pe;
-    grn_content_type ot;
+    grn_content_type ot = GRN_CONTENT_NONE;
     grn_timeval_now(ctx, &ctx->impl->tv);
     GRN_LOG(ctx, GRN_LOG_NONE, "%08x|>%.*s", (intptr_t)ctx, path_len, path);
     GRN_TEXT_INIT(&key, 0);
@@ -1300,66 +1311,129 @@ grn_cache_fin(void)
 
 #define ALIGN_SIZE (1<<3)
 #define ALIGN_MASK (ALIGN_SIZE-1)
+#define GRN_CTX_ALLOC_CLEAR 1
 
 void *
 grn_ctx_alloc(grn_ctx *ctx, size_t size, int flags,
               const char* file, int line, const char *func)
 {
-  if (!ctx) { return NULL; }
+  void *res = NULL;
+  if (!ctx) { return res; }
   if (!ctx->impl) {
     grn_ctx_impl_init(ctx);
-    if (ERRP(ctx, GRN_ERROR)) { return NULL; }
+    if (ERRP(ctx, GRN_ERROR)) { return res; }
   }
+  CRITICAL_SECTION_ENTER(ctx->impl->lock);
   {
     int32_t i;
-    uint64_t *header;
+    int32_t *header;
     grn_io_mapinfo *mi;
     size = ((size + ALIGN_MASK) & ~ALIGN_MASK) + ALIGN_SIZE;
-    if (size > SEGMENT_SIZE) {
+    if (size > GRN_CTX_SEGMENT_SIZE) {
       uint64_t npages = (size + (grn_pagesize - 1)) / grn_pagesize;
       if (npages >= (1LL<<32)) {
         MERR("too long request size=%zu", size);
-        return NULL;
+        goto exit;
       }
       for (i = 0, mi = ctx->impl->segs;; i++, mi++) {
-        if (i >= N_SEGMENTS) {
+        if (i >= GRN_CTX_N_SEGMENTS) {
           MERR("all segments are full");
-          return NULL;
+          goto exit;
         }
         if (!mi->map) { break; }
       }
-      if (!grn_io_anon_map(ctx, mi, npages * grn_pagesize)) { return NULL; }
+      if (!grn_io_anon_map(ctx, mi, npages * grn_pagesize)) { goto exit; }
       //GRN_LOG(ctx, GRN_LOG_NOTICE, "map i=%d (%d)", i, npages * grn_pagesize);
       mi->nref = (uint32_t) npages;
-      mi->count = SEGMENT_VLEN;
+      mi->count = GRN_CTX_SEGMENT_VLEN;
       ctx->impl->currseg = -1;
       header = mi->map;
-      *header = (uint64_t) i;
-      return &header[1];
+      header[0] = i;
+      header[1] = (int32_t) size;
     } else {
       i = ctx->impl->currseg;
       mi = &ctx->impl->segs[i];
-      if (i < 0 || size + mi->nref > SEGMENT_SIZE) {
+      if (i < 0 || size + mi->nref > GRN_CTX_SEGMENT_SIZE) {
         for (i = 0, mi = ctx->impl->segs;; i++, mi++) {
-          if (i >= N_SEGMENTS) {
+          if (i >= GRN_CTX_N_SEGMENTS) {
             MERR("all segments are full");
-            return NULL;
+            goto exit;
           }
           if (!mi->map) { break; }
         }
-        if (!grn_io_anon_map(ctx, mi, SEGMENT_SIZE)) { return NULL; }
+        if (!grn_io_anon_map(ctx, mi, GRN_CTX_SEGMENT_SIZE)) { goto exit; }
         //GRN_LOG(ctx, GRN_LOG_NOTICE, "map i=%d", i);
         mi->nref = 0;
-        mi->count = SEGMENT_WORD;
+        mi->count = GRN_CTX_SEGMENT_WORD;
         ctx->impl->currseg = i;
       }
-      header = (uint64_t *)((byte *)mi->map + mi->nref);
+      header = (int32_t *)((byte *)mi->map + mi->nref);
       mi->nref += size;
       mi->count++;
-      *header = (uint64_t) i;
-      return &header[1];
+      header[0] = i;
+      header[1] = (int32_t) size;
+      if ((flags & GRN_CTX_ALLOC_CLEAR) && (mi->count & GRN_CTX_SEGMENT_DIRTY)) {
+        memset(&header[2], 0, size);
+      }
+    }
+    /*
+    {
+      char g = (ctx == &grn_gctx) ? 'g' : ' ';
+      GRN_LOG(ctx, GRN_LOG_NOTICE, "+%c(%p) %s:%d(%s) (%d:%d)%p mi(%d:%d)", g, ctx, file, line, func, header[0], header[1], &header[2], mi->nref, (mi->count & GRN_CTX_SEGMENT_MASK));
+    }
+    */
+    res = &header[2];
+  }
+exit :
+  CRITICAL_SECTION_LEAVE(ctx->impl->lock);
+  return res;
+}
+
+void *
+grn_ctx_malloc(grn_ctx *ctx, size_t size,
+              const char* file, int line, const char *func)
+{
+  return grn_ctx_alloc(ctx, size, 0, file, line, func);
+}
+
+void *
+grn_ctx_calloc(grn_ctx *ctx, size_t size,
+              const char* file, int line, const char *func)
+{
+  return grn_ctx_alloc(ctx, size, GRN_CTX_ALLOC_CLEAR, file, line, func);
+}
+
+void *
+grn_ctx_realloc(grn_ctx *ctx, void *ptr, size_t size,
+                const char* file, int line, const char *func)
+{
+  void *res = NULL;
+  if (size) {
+    /* todo : expand if possible */
+    res = grn_ctx_alloc(ctx, size, 0, file, line, func);
+    if (res && ptr) {
+      int32_t *header = &((int32_t *)ptr)[-2];
+      size_t size_ = header[1];
+      memcpy(res, ptr, size_ > size ? size : size_);
+      grn_ctx_free(ctx, ptr, file, line, func);
+    }
+  } else {
+    grn_ctx_free(ctx, ptr, file, line, func);
+  }
+  return res;
+}
+
+char *
+grn_ctx_strdup(grn_ctx *ctx, const char *s, const char* file, int line, const char *func)
+{
+  void *res = NULL;
+  if (s) {
+    size_t size = strlen(s) + 1;
+    if ((res = grn_ctx_alloc(ctx, size, 0, file, line, func))) {
+      memcpy(res, s, size);
     }
   }
+  return res;
 }
 
 void
@@ -1371,19 +1445,31 @@ grn_ctx_free(grn_ctx *ctx, void *ptr,
     ERR(GRN_INVALID_ARGUMENT,"ctx without impl passed.");
     return;
   }
-  {
-    uint64_t *header = &((uint64_t *)ptr)[-1];
-    if (*header >= N_SEGMENTS) {
+  CRITICAL_SECTION_ENTER(ctx->impl->lock);
+  if (ptr) {
+    int32_t *header = &((int32_t *)ptr)[-2];
+
+    if (header[0] >= GRN_CTX_N_SEGMENTS) {
       ERR(GRN_INVALID_ARGUMENT,"invalid ptr passed. ptr=%p seg=%zu", ptr, *header);
-      return;
+      goto exit;
     }
+    /*
     {
-      int32_t i = (int32_t)*header;
+      int32_t i = header[0];
+      char c = 'X', g = (ctx == &grn_gctx) ? 'g' : ' ';
       grn_io_mapinfo *mi = &ctx->impl->segs[i];
-      if (mi->count & SEGMENT_VLEN) {
+      if (!(mi->count & GRN_CTX_SEGMENT_VLEN) &&
+          mi->map <= (void *)header && (char *)header < ((char *)mi->map + GRN_CTX_SEGMENT_SIZE)) { c = '-'; }
+      GRN_LOG(ctx, GRN_LOG_NOTICE, "%c%c(%p) %s:%d(%s) (%d:%d)%p mi(%d:%d)", c, g, ctx, file, line, func, header[0], header[1], &header[2], mi->nref, (mi->count & GRN_CTX_SEGMENT_MASK));
+    }
+    */
+    {
+      int32_t i = header[0];
+      grn_io_mapinfo *mi = &ctx->impl->segs[i];
+      if (mi->count & GRN_CTX_SEGMENT_VLEN) {
         if (mi->map != header) {
           ERR(GRN_INVALID_ARGUMENT,"invalid ptr passed.. ptr=%p seg=%d", ptr, i);
-          return;
+          goto exit;
         }
         //GRN_LOG(ctx, GRN_LOG_NOTICE, "umap i=%d (%d)", i, mi->nref * grn_pagesize);
         grn_io_anon_unmap(ctx, mi, mi->nref * grn_pagesize);
@@ -1391,22 +1477,24 @@ grn_ctx_free(grn_ctx *ctx, void *ptr,
       } else {
         if (!mi->map) {
           ERR(GRN_INVALID_ARGUMENT,"invalid ptr passed... ptr=%p seg=%d", ptr, i);
-          return;
+          goto exit;
         }
         mi->count--;
-        if (!(mi->count & SEGMENT_MASK)) {
+        if (!(mi->count & GRN_CTX_SEGMENT_MASK)) {
           //GRN_LOG(ctx, GRN_LOG_NOTICE, "umap i=%d", i);
           if (i == ctx->impl->currseg) {
-            memset(mi->map, 0, mi->nref);
+            mi->count |= GRN_CTX_SEGMENT_DIRTY;
             mi->nref = 0;
           } else {
-            grn_io_anon_unmap(ctx, mi, SEGMENT_SIZE);
+            grn_io_anon_unmap(ctx, mi, GRN_CTX_SEGMENT_SIZE);
             mi->map = NULL;
           }
         }
       }
     }
   }
+exit :
+  CRITICAL_SECTION_LEAVE(ctx->impl->lock);
 }
 
 #define DB_P(s) ((s) && (s)->header.type == GRN_DB)
@@ -1435,7 +1523,7 @@ grn_ctx_use(grn_ctx *ctx, grn_obj *db)
 }
 
 void *
-grn_ctx_alloc_lifo(grn_ctx *ctx, size_t size, int flags,
+grn_ctx_alloc_lifo(grn_ctx *ctx, size_t size,
                    const char* file, int line, const char *func)
 {
   if (!ctx) { return NULL; }
@@ -1446,14 +1534,14 @@ grn_ctx_alloc_lifo(grn_ctx *ctx, size_t size, int flags,
   {
     int32_t i = ctx->impl->lifoseg;
     grn_io_mapinfo *mi = &ctx->impl->segs[i];
-    if (size > SEGMENT_SIZE) {
+    if (size > GRN_CTX_SEGMENT_SIZE) {
       uint64_t npages = (size + (grn_pagesize - 1)) / grn_pagesize;
       if (npages >= (1LL<<32)) {
         MERR("too long request size=%zu", size);
         return NULL;
       }
       for (;;) {
-        if (++i >= N_SEGMENTS) {
+        if (++i >= GRN_CTX_N_SEGMENTS) {
           MERR("all segments are full");
           return NULL;
         }
@@ -1462,22 +1550,22 @@ grn_ctx_alloc_lifo(grn_ctx *ctx, size_t size, int flags,
       }
       if (!grn_io_anon_map(ctx, mi, npages * grn_pagesize)) { return NULL; }
       mi->nref = (uint32_t) npages;
-      mi->count = SEGMENT_VLEN|SEGMENT_LIFO;
+      mi->count = GRN_CTX_SEGMENT_VLEN|GRN_CTX_SEGMENT_LIFO;
       ctx->impl->lifoseg = i;
       return mi->map;
     } else {
       size = (size + ALIGN_MASK) & ~ALIGN_MASK;
-      if (i < 0 || (mi->count & SEGMENT_VLEN) || size + mi->nref > SEGMENT_SIZE) {
+      if (i < 0 || (mi->count & GRN_CTX_SEGMENT_VLEN) || size + mi->nref > GRN_CTX_SEGMENT_SIZE) {
         for (;;) {
-          if (++i >= N_SEGMENTS) {
+          if (++i >= GRN_CTX_N_SEGMENTS) {
             MERR("all segments are full");
             return NULL;
           }
           if (!(++mi)->map) { break; }
         }
-        if (!grn_io_anon_map(ctx, mi, SEGMENT_SIZE)) { return NULL; }
+        if (!grn_io_anon_map(ctx, mi, GRN_CTX_SEGMENT_SIZE)) { return NULL; }
         mi->nref = 0;
-        mi->count = SEGMENT_WORD|SEGMENT_LIFO;
+        mi->count = GRN_CTX_SEGMENT_WORD|GRN_CTX_SEGMENT_LIFO;
         ctx->impl->lifoseg = i;
       }
       {
@@ -1506,9 +1594,9 @@ grn_ctx_free_lifo(grn_ctx *ctx, void *ptr,
       return;
     }
     for (; i >= 0; i--, mi--) {
-      if (!(mi->count & SEGMENT_LIFO)) { continue; }
+      if (!(mi->count & GRN_CTX_SEGMENT_LIFO)) { continue; }
       if (done) { break; }
-      if (mi->count & SEGMENT_VLEN) {
+      if (mi->count & GRN_CTX_SEGMENT_VLEN) {
         if (mi->map == ptr) { done = 1; }
         grn_io_anon_unmap(ctx, mi, mi->nref * grn_pagesize);
         mi->map = NULL;
@@ -1521,7 +1609,7 @@ grn_ctx_free_lifo(grn_ctx *ctx, void *ptr,
             break;
           }
         }
-        grn_io_anon_unmap(ctx, mi, SEGMENT_SIZE);
+        grn_io_anon_unmap(ctx, mi, GRN_CTX_SEGMENT_SIZE);
         mi->map = NULL;
       }
     }
@@ -1666,13 +1754,13 @@ grn_calloc_default(grn_ctx *ctx, size_t size, const char* file, int line, const
 }
 
 void
-grn_free(grn_ctx *ctx, void *ptr, const char* file, int line)
+grn_free_default(grn_ctx *ctx, void *ptr, const char* file, int line, const char *func)
 {
   if (!ctx) { return; }
   {
     free(ptr);
     if (ptr) {
-        GRN_ADD_ALLOC_COUNT(-1);
+      GRN_ADD_ALLOC_COUNT(-1);
     } else {
       GRN_LOG(ctx, GRN_LOG_ALERT, "free fail (%p) (%s:%d) <%d>", ptr, file, line, alloc_count);
     }
