@@ -85,7 +85,8 @@ setup_data(void)
 {
   assert_send_command("load "
                       "'[[\"_key\",\"content\"],"
-                      "[\"ボロ\",\"うちのボロTV（アナログ...）はまだ現役です\"]]' "
+                      "[\"ボロ\",\"うちのボロTV（アナログ...）はまだ現役です\"],"
+                      "[\"ロボ\",\"ロボット 鉄\"]]'"
                       "Comments");
 }
 
@@ -180,7 +181,7 @@ query(const gchar *string)
 void
 test_japanese_parenthesis(void)
 {
-  cut_omit("token->uni_symbol sould be customzable");
+  cut_omit("token->uni_symbol should be customzable");
   cut_assert_not_null(grn_table_select(context, comments,
                                        query("content:@）は"),
                                        result, GRN_OP_OR));
@@ -201,3 +202,15 @@ test_japanese_dot_parenthesis(void)
                          result,
                          "_key");
 }
+
+void
+test_bigram_end_with_space_and_single_char(void)
+{
+  cut_assert_not_null(grn_table_select(context, comments,
+                                       query("content:@\"ロボット 鉄\""),
+                                       result, GRN_OP_OR));
+  grn_test_assert_select(context,
+                         gcut_take_new_list_string("ロボ", NULL),
+                         result,
+                         "_key");
+}
