@@ -37,6 +37,27 @@
       __VA_ARGS__),                                             \
     grn_test_assert_equal_rc(expected, actual))
 
+#define grn_test_assert_equal_id(context, expected, actual, ...)        \
+  cut_trace_with_info_expression(                                       \
+    cut_test_with_user_message(                                         \
+      grn_test_assert_equal_id_helper((context),                        \
+                                      (expected), (actual),             \
+                                      #context,                         \
+                                      #expected, #actual),              \
+      __VA_ARGS__),                                                     \
+    grn_test_assert_equal_id(context, expected, actual))
+
+#define grn_test_assert_equal_record_id(context, table,                 \
+                                        expected, actual, ...)          \
+  cut_trace_with_info_expression(                                       \
+    cut_test_with_user_message(                                         \
+      grn_test_assert_equal_record_id_helper((context), (table),        \
+                                             (expected), (actual),      \
+                                             #context, #table,          \
+                                             #expected, #actual),       \
+      __VA_ARGS__),                                                     \
+    grn_test_assert_equal_record_id(context, table, expected, actual))
+
 #define grn_test_assert_nil(expression, ...)                    \
   cut_trace_with_info_expression(                               \
     cut_test_with_user_message(                                 \
@@ -137,6 +158,21 @@ void     grn_test_assert_equal_rc_helper(grn_rc       expected,
                                          grn_rc       actual,
                                          const gchar *expression_expected,
                                          const gchar *expression_actual);
+void     grn_test_assert_equal_id_helper(grn_ctx     *context,
+                                         grn_id       expected,
+                                         grn_id       actual,
+                                         const gchar *expression_context,
+                                         const gchar *expression_expected,
+                                         const gchar *expression_actual);
+void     grn_test_assert_equal_record_id_helper
+                                        (grn_ctx     *context,
+                                         grn_obj     *table,
+                                         grn_id       expected,
+                                         grn_id       actual,
+                                         const gchar *expression_context,
+                                         const gchar *expression_table,
+                                         const gchar *expression_expected,
+                                         const gchar *expression_actual);
 void     grn_test_assert_nil_helper     (grn_id       id,
                                          const gchar *expression);
 void     grn_test_assert_not_nil_helper (grn_id       id,
